"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var errors_1 = tslib_1.__importDefault(require("../errors"));
var utils_1 = require("../utils");
var DatabaseSession = /** @class */ (function () {
    function DatabaseSession(db, state, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this.db = utils_1.ensureParamObject("db", db);
        this.state = utils_1.ensureParamObject("state", state);
        var _a = options.readOnly, readOnly = _a === void 0 ? false : _a, _b = options.tableSchemas, tableSchemas = _b === void 0 ? db.tables : _b;
        this.readOnly = readOnly;
        this.tables = utils_1.toObject(tableSchemas.map(function (tableSchema) {
            return _this.db.factory.newTableModel(_this, tableSchema, state[tableSchema.name]);
        }), function (t) { return t.schema.name; });
    }
    DatabaseSession.prototype.getTable = function (name) {
        return this.tables[name];
    };
    DatabaseSession.prototype.upsert = function (ctx) {
        var _this = this;
        if (this.readOnly)
            throw new Error(errors_1.default.sessionReadonly());
        Object.keys(ctx.output).forEach(function (name) {
            if (name !== ctx.schema.name)
                _this.tables[name].upsertNormalized(ctx.output[name]);
        });
        Object.keys(ctx.emits).forEach(function (name) {
            if (name !== ctx.schema.name)
                _this.tables[name].upsert(ctx.emits[name]);
        });
    };
    DatabaseSession.prototype.commit = function () {
        var _this = this;
        if (this.readOnly)
            throw new Error(errors_1.default.sessionReadonly());
        Object.keys(this.tables).forEach(function (table) {
            var _a;
            var oldState = _this.state[table];
            var newState = _this.tables[table].state;
            if (oldState !== newState)
                _this.state = tslib_1.__assign({}, _this.state, (_a = {}, _a[table] = newState, _a));
        });
        return this.state;
    };
    return DatabaseSession;
}());
exports.default = DatabaseSession;
