import { NormalizeOptions, PartialValue, PartialValues, Session, Table, TableRecord, TableSchema, TableState, Values, ValueType } from "../types";
import NormalizeContext from "./NormalizeContext";
export default class TableModel<R extends TableRecord> implements Table<R> {
    readonly session: Session;
    readonly schema: TableSchema;
    state: TableState<ValueType<R>>;
    dirty: boolean;
    constructor(session: Session, schema: TableSchema, state?: TableState);
    readonly length: number;
    all(): R[];
    values(): ValueType<R>[];
    exists(id: number | string): boolean;
    get(id: number | string): R;
    getOrDefault(id: number | string): R | undefined;
    getValue(id: number | string): ValueType<R> | undefined;
    getIndex(name: string, fk: string): string[];
    insert(data: Values<R>, argument?: any): string[];
    update(data: PartialValues<R>, argument?: any): string[];
    upsert(data: PartialValues<R>, argument?: any): string[];
    delete(data: string | number | PartialValue<R> | (string | number | PartialValue<R>)[]): number;
    deleteAll(): void;
    upsertNormalized(norm: TableState<ValueType<R>>): void;
    insertNormalized(table: TableState<ValueType<R>>, updateIndexes?: boolean): void;
    updateNormalized(table: TableState<ValueType<R>>, updateIndexes?: boolean): void;
    normalize(data: PartialValues<R>, options?: NormalizeOptions): NormalizeContext;
    private _normalizedAction;
    private _cleanIndexes;
    private _updateIndexes;
    private _deleteCascade;
}
