import * as tslib_1 from "tslib";
import { RESERVED_PROPERTIES } from "./constants";
import errors from "./errors";
import * as utils from "./utils";
export var createRecordModelClass = function (Base) {
    return /** @class */ (function (_super) {
        tslib_1.__extends(ExtendedRecordModel, _super);
        function ExtendedRecordModel(id, table) {
            var _this = _super.call(this, id, table) || this;
            _this._fields = {};
            return _this;
        }
        return ExtendedRecordModel;
    }(Base));
};
var RecordModelFactory = /** @class */ (function () {
    function RecordModelFactory(recordBaseType, factory) {
        this._recordClass = {};
        this._defineProperty = function (model, name, field, factory, cache) {
            if (cache === void 0) { cache = true; }
            if (RESERVED_PROPERTIES.indexOf(name) >= 0)
                throw new Error(errors.reservedProperty(field.table.name, name));
            Object.defineProperty(model.prototype, name, {
                get: function () {
                    // TODO: Improve the instance cache mechanism. Invalidate when the field value changes..
                    return cache
                        ? (this._fields[name] || (this._fields[name] = factory(field, this)))
                        : factory(field, this);
                }
            });
        };
        this._recordBaseType = recordBaseType;
        this._factory = factory;
    }
    RecordModelFactory.prototype.newRecordModel = function (id, table) {
        utils.ensureParamString("id", id);
        utils.ensureParamObject("table", table);
        var model = this.createRecordModel(table.schema);
        return new model(id, table);
    };
    RecordModelFactory.prototype.createRecordModel = function (schema) {
        var _this = this;
        if (this._recordClass[schema.name])
            return this._recordClass[schema.name];
        else {
            var model_1 = createRecordModelClass(this._recordBaseType);
            schema.fields.forEach(function (f) {
                return (f.isForeignKey || !f.isPrimaryKey)
                    && _this._defineProperty(model_1, f.propName, f, _this._newRecordField.bind(_this), false);
            });
            schema.relations.forEach(function (f) {
                return f.relationName && _this._defineProperty(model_1, f.relationName, f, f.unique
                    ? _this._newRecordRelation.bind(_this)
                    : _this._newRecordSet.bind(_this), !f.unique);
            });
            return this._recordClass[schema.name] = model_1;
        }
    };
    RecordModelFactory.prototype._newRecordField = function (schema, record) {
        if (!schema.isForeignKey)
            return this._factory.newRecordFieldModel(schema, record);
        var refTable = schema.references && record.table.session.tables[schema.references];
        if (!refTable)
            throw new Error(errors.fkReferenceNotInSession(schema.name, schema.references));
        var recordId = schema.getRecordValue(record);
        if (recordId === undefined)
            return null;
        return refTable.get(recordId);
    };
    RecordModelFactory.prototype._newRecordSet = function (schema, record) {
        var refTable = record.table.session.tables[schema.table.name];
        if (!refTable)
            throw new Error(errors.tableNotInSession(schema.table.name));
        return this._factory.newRecordSetModel(refTable, schema, record);
    };
    RecordModelFactory.prototype._newRecordRelation = function (schema, record) {
        var refTable = record.table.session.tables[schema.table.name];
        if (!refTable)
            throw new Error(errors.tableNotInSession(schema.table.name));
        var id = refTable.getIndex(schema.name, record.id)[0];
        if (id === undefined)
            return null;
        return this.newRecordModel(id, refTable);
    };
    return RecordModelFactory;
}());
export { RecordModelFactory };
var createModelFactory = function (factory, RecordModel) {
    var recordFactory = new RecordModelFactory(RecordModel, factory);
    return tslib_1.__assign({}, factory, { newRecordModel: function (id, table) { return recordFactory.newRecordModel(id, table); } });
};
export default createModelFactory;
