import { TYPE_ATTR, TYPE_MODIFIED, TYPE_PK } from "../constants";
import errors from "../errors";
import { ensureParamFunction, ensureParamObject, ensureParamString, optionalParamString } from "../utils";
var FieldSchemaModel = /** @class */ (function () {
    function FieldSchemaModel(table, name, schema, cascadeAsDefault) {
        if (cascadeAsDefault === void 0) { cascadeAsDefault = false; }
        this.table = ensureParamObject("table", table);
        ensureParamString("name", name);
        ensureParamObject("schema", schema);
        this.type = schema.type || TYPE_ATTR;
        this.name = optionalParamString("schema.fieldName", schema.fieldName, name);
        this.propName = optionalParamString("schema.propName", schema.propName, name);
        this.references = optionalParamString("schema.references", schema.references);
        this.isPrimaryKey = schema.pk === true || schema.type === TYPE_PK;
        this.isForeignKey = !!this.references;
        this.isStamp = schema.stamp === true || schema.type === TYPE_MODIFIED;
        this._valueFactory = schema.value !== undefined
            ? ensureParamFunction("schema.value", schema.value).bind(this)
            : undefined;
        if (this.isPrimaryKey || this.isForeignKey) {
            if (this.isForeignKey)
                this.relationName = optionalParamString("schema.relationName", schema.relationName);
            this.cascade = schema.cascade === undefined ? cascadeAsDefault : schema.cascade === true;
            this.unique = schema.unique === true;
            // not null is default true, for PK's and FK's
            this.notNull = schema.notNull === undefined ? true : schema.notNull === true;
        }
        else {
            this.cascade = false;
            this.unique = false;
            this.notNull = schema.notNull === true;
        }
    }
    Object.defineProperty(FieldSchemaModel.prototype, "refTable", {
        get: function () { return this._refTable; },
        enumerable: true,
        configurable: true
    });
    FieldSchemaModel.prototype.connect = function (schemas) {
        if (this.references) {
            this._refTable = schemas[this.references];
            if (!this._refTable)
                throw new Error(errors.fkInvalidReference(this.table.name, this.name, this.references));
        }
    };
    FieldSchemaModel.prototype.getValue = function (data, record) {
        return this._valueFactory ? this._valueFactory(data, {
            record: record,
            schema: this
        }) : data[this.name];
    };
    FieldSchemaModel.prototype.getRecordValue = function (record) {
        return this.getValue(record.value, record);
    };
    return FieldSchemaModel;
}());
export default FieldSchemaModel;
