import errors from "../errors";
import { ensureParamObject } from "../utils";
var RecordSetModel = /** @class */ (function () {
    function RecordSetModel(table, schema, owner) {
        this.table = ensureParamObject("table", table);
        this.schema = ensureParamObject("schema", schema);
        this.owner = ensureParamObject("owner", owner, "id");
    }
    Object.defineProperty(RecordSetModel.prototype, "ids", {
        get: function () {
            return this.table.getIndex(this.schema.name, this.owner.id);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RecordSetModel.prototype, "length", {
        get: function () {
            return this.ids.length;
        },
        enumerable: true,
        configurable: true
    });
    RecordSetModel.prototype.all = function () {
        var _this = this;
        return this.ids.map(function (id) { return _this.table.get(id); });
    };
    RecordSetModel.prototype.values = function () {
        var _this = this;
        return this.ids.map(function (id) {
            var value = _this.table.getValue(id);
            if (!value)
                throw new Error(errors.recordNotFound(_this.table.schema.name, id));
            return value;
        });
    };
    RecordSetModel.prototype.add = function (data) {
        return this.table.insert(this._normalize(data));
    };
    RecordSetModel.prototype.remove = function (data) {
        var _this = this;
        var ids = this._normalize(data).map(function (obj) {
            return _this.table.schema.ensurePrimaryKey(obj);
        });
        return this.table.delete(ids);
    };
    RecordSetModel.prototype.update = function (data) {
        return this.table.update(this._normalize(data));
    };
    RecordSetModel.prototype.delete = function () {
        this.table.delete(this.ids);
    };
    RecordSetModel.prototype._normalize = function (data) {
        return this.table.schema.inferRelations(data, this.schema, this.owner.id);
    };
    return RecordSetModel;
}());
export default RecordSetModel;
