import * as tslib_1 from "tslib";
import errors from "./errors";
import { mergeIds } from "./utils";
export var merge = function (original, modified) {
    var ids = modified.ids
        ? mergeIds(original.ids, modified.ids, true)
        : original.ids;
    var byId = modified.byId
        ? tslib_1.__assign({}, original.byId, modified.byId) :
        original.byId;
    return tslib_1.__assign({}, original, { byId: byId,
        ids: ids });
};
export var splice = function (original, idsToDelete) {
    var byId = tslib_1.__assign({}, original.byId);
    var ids = original.ids.slice();
    var indexes = tslib_1.__assign({}, original.indexes);
    var deleted = idsToDelete.reduce(function (n, id) {
        var record = byId[id];
        delete byId[id];
        var idx = ids.indexOf(id);
        if (idx >= 0)
            ids.splice(idx, 1);
        if (record)
            return n.concat([{ id: id, record: record }]);
        return n;
    }, []);
    return {
        deleted: deleted,
        state: tslib_1.__assign({}, original, { byId: byId, ids: ids, indexes: indexes })
    };
};
export var updateIndexes = function (name, original, modified) {
    Object.keys(modified.indexes).forEach(function (key) {
        var idx = original.indexes[key]
            || (original.indexes[key] = { unique: modified.indexes[key].unique, values: {} });
        Object.keys(modified.indexes[key].values).forEach(function (fk) {
            var idxBucket = idx.values[fk] || (idx.values[fk] = []);
            var modifiedBucket = mergeIds(idxBucket, modified.indexes[key].values[fk], false);
            if (idx.unique && modifiedBucket.length > 1)
                throw new Error(errors.fkViolation(name, key));
            idx.values[fk] = modifiedBucket;
        });
    });
};
export var cleanIndexes = function (keys, id, indexes) {
    keys.forEach(function (fk) {
        var fkIdx = -1;
        if (fk.value && indexes[fk.name] && indexes[fk.name].values[fk.value])
            fkIdx = indexes[fk.name].values[fk.value].indexOf(id);
        if (fkIdx >= 0) {
            var idxBucket = indexes[fk.name].values[fk.value].slice();
            idxBucket.splice(fkIdx, 1);
            indexes[fk.name].values[fk.value] = idxBucket;
        }
        else if (indexes[fk.name]) {
            delete indexes[fk.name].values[id];
            if (Object.keys(indexes[fk.name].values).length === 0)
                delete indexes[fk.name];
        }
    });
};
export default {
    cleanIndexes: cleanIndexes,
    merge: merge,
    splice: splice,
    updateIndexes: updateIndexes
};
